/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#ifndef CMMSPECIFICATIONSHEADERFILE
#define CMMSPECIFICATIONSHEADERFILE

#include <QMainWindow>

#include "databaseitem.h"

class QAction;
class QListWidget;
class QTreeWidget;
class QTreeWidgetItem;
class QSettings;
class QCloseEvent;
class TDatabase;
class TMessageBox;
class TImportExportThread;
class TProgressDialog;
class TSpecificationEditorDialog;
class TSpecificationLimitsDialog;
class TExportOptionsDialog;

class TCMMSpecifications : public QMainWindow
{
Q_OBJECT

public:

// CREATORS
	TCMMSpecifications(const QWidget *parent=0, Qt::WindowFlags flags=Qt::WindowFlags());
	virtual ~TCMMSpecifications(void);

// ACCESSORS

// MANIPULATORS

signals:

private slots:
	
	void New_Database(void);
	void Open_Database(void);
	void Close_Database(void);
	void Export(void);
	void Import(void);
	void Output(void);
	
	void Edit_Specification(void);
	void Specification_Limits(void);
	
	void Delete_Make(void);
	void Delete_Model(void);
	void Delete_Option(void);
	
	void Help_About(void);
	
	void Make_Selection_Changed(void);
	void Model_Selection_Changed(void);
	void Option_Selection_Changed(void);

	void Thread_Started(void);
	void Thread_Finished(void);
	void Thread_Progress(int,int);		// value, range
	void Thread_Progress_Text(const QString&);	// description of sequence
	
protected:
	virtual void closeEvent(QCloseEvent *event);

private:
	QAction								*d_new_database_action;
	QAction								*d_open_database_action;
	QAction								*d_close_database_action;
	QAction								*d_export_action;
	QAction								*d_import_action;
	QAction								*d_output_action;
	QAction								*d_quit_action;
	QAction								*d_delete_make_action;
	QAction								*d_delete_model_action;
	QAction								*d_delete_option_action;
	QAction								*d_edit_specifications_action;
	QAction								*d_limits_action;
	QListWidget							*d_make_list;
	QListWidget							*d_model_list;
	QListWidget							*d_option_list;
	QTreeWidget							*d_specification_tree;
	
	TProgressDialog						*d_progress_dialog;
	TSpecificationEditorDialog			*d_specification_editor_dialog;
	TSpecificationLimitsDialog			*d_specification_limits_dialog;
	TExportOptionsDialog				*d_export_options_dialog;
	
	TImportExportThread					*d_import_export_thread;
	
	TDatabase							*d_database;
	TMessageBox							*d_msg_box;
	
	QSettings							*d_settings;
	QString								d_application_path;
	QString								d_data_path;
	QString								d_current_path;

	bool Load_Database(const QString &database_file);
	
	void Set_Selection(const QString &make,const QString &model,const QString &option);
	void Create_Specification_Entry(const QString &standard,const TDatabaseItem &specification);
	

// NOT IMPLEMENTED
	TCMMSpecifications(const TCMMSpecifications&);
	TCMMSpecifications& operator=(const TCMMSpecifications&);
};

#endif
