/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef DATABASEHEADERFILE
#define DATABASEHEADERFILE

#include <QString>
#include <QStringList>
#include <vector>

#include "databaseitem.h"

class QSqlDatabase;

class TDatabase
{
public:
	
// CREATORS
	TDatabase(void);
	~TDatabase(void);
	
// ACCESSORS
	bool Is_Connected(void) const;
	
	bool CMM_Make_Exists(const QString &make) const;
	bool CMM_Model_Exists(const QString &make,const QString &model) const;
	bool CMM_Option_Exists(const QString &make,const QString &model,const QString &option) const;
	bool Standard_Type_Exists(const QString &standard_type) const;
	bool Component_Type_Exists(const QString &standard_type,const QString &component_type) const;
	
	QStringList CMM_Make_List(bool * const valid) const;
	QStringList CMM_Model_List(const QString &make,bool * const valid) const;
	QStringList CMM_Option_List(const QString &make,const QString &model,bool * const valid) const;
	QStringList Standard_Type_List(bool * const valid) const;
	QStringList Component_Type_List(const QString &standard_type,bool * const valid) const;
	
	QString Component_Description(const QString &standard_type,const QString &component_name) const;
	int Component_Expression_Types(const QString &standard_type,const QString &component_name) const;

	TDatabaseItem Database_Item(const QString &make,const QString &model,const QString &option,const QString &standard_type,bool * const valid) const;
	
	QString Last_Error(void) const {return d_last_error;}
		
// MANIPULATORS
	bool Connect(const QString &file_name);
	bool Initialize(void);								// verifies required tables exist
	void Disconnect(void);
	void Clear(void);

	bool Add_Database_Item(const TDatabaseItem &item);
	bool Create_Estimated_Specifications(const QString &make,const QString &model,const QString &option);
	
	bool Delete_Make(const QString &make);
	bool Delete_Model(const QString &make,const QString &model);
	bool Delete_Option(const QString &make,const QString &model,const QString &option);
	
private:
	
	QString								d_database_instance_name;
	QString								d_database_file_name;
	
	mutable QSqlDatabase				*d_database;
	mutable QString						d_last_error;
	
	bool Initialize_Default_Standard_Types(void);
	bool Add_Standard_Type(const QString &standard);

	bool Initialize_Default_Component_Types(void);
	bool Add_Component_Type(const int standard_id,const QString &component,const QString &description,const int expression_types);
	
	bool Model_Type_Exists(const QString &model) const;
	bool Option_Type_Exists(const QString &option) const;
	
	int Make_Id(const QString &make) const;
	int Model_Type_Id(const QString &model) const;
	int Option_Type_Id(const QString &option) const;
	int Standard_Type_Id(const QString &standard_type) const;
	
	QString Make(const int make_id) const;
	QString Model_Type(const int model_type_id) const;
	QString Option_Type(const int option_type_id) const;
	QString Standard_Type(const int standard_type_id) const;
	
	int Component_Type_Id(const int standard_type_id,const QString &component_type) const;
	QString Component_Name(const int standard_type_id,const int component_id) const;
	QString Component_Description(const int standard_type_id,const int component_id) const;
	int Component_Expression_Types(const int standard_type_id,const int component_id) const;

	bool Specification_Exists(const int make_id,const int model_id, const int option_type_id,const int standard_type_id,const int component_id) const;
	TDatabaseItem Database_Item(const int make_id,const int model_id, const int option_type_id,const int standard_type_id,bool * const valid) const;
	
	bool Remove_Estimated_Specifications(const int make_id,const int model_id,const int option_type_id);
	bool Create_Estimated_Specifications(const int make_id,const int model_id,const int option_type_id,const QString &standard_type);
	

// NOT IMPLEMENTED
	TDatabase(const TDatabase&);
	TDatabase& operator=(const TDatabase&);
};


#endif
