/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#include <QApplication>
#include <QFrame>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QSpacerItem>
#include <cmath>

#include "expressionitem.h"

#include "specificationlimitsdialog.h"

TSpecificationLimitsDialog::TSpecificationLimitsDialog(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QDialog(const_cast<QWidget*>(parent),flags)
{
	QFrame								*separator_hline_1;
	QFrame								*separator_hline_2;
	QFrame								*separator_hline_3;
	QGridLayout							*dialog_layout;
	QHBoxLayout							*button_hlayout;
	QLabel								*b89_repeat_label;
	QLabel								*b89_repeat_description_label;
	QLabel								*ballbar_range_description_label;
	QLabel								*ballbar_range_label;
	QLabel								*cmma_m_length_description_label;
	QLabel								*cmma_m_length_label;
	QLabel								*cmma_repeat_description_label;
	QLabel								*cmma_repeat_label;
	QLabel								*iso_e0_length_description_label;
	QLabel								*iso_e0_length_label;
	QLabel								*iso_e150_length_description_label;
	QLabel								*iso_e150_length_label;
	QLabel								*iso_r0_repeat_description_label;
	QLabel								*iso_r0_repeat_label;
	QLabel								*length_label;
	QLabel								*length_units_label;
	QLabel								*make_label;
	QLabel								*model_label;
	QLabel								*option_label;
	QLabel								*temperature_label;
	QLabel								*temperature_units_label;
	QLabel								*vdivde_u3_length_label;
	QLabel								*vdivde_v3_repeat_description_label;
	QLabel								*vdivde_v3_repeat_label;
	QLabel								*vdivde_u3_length_description_label;
	QPushButton							*close_button;
	QSpacerItem							*button_hspacer;
	QSpacerItem							*dialog_vspacer;

	this->resize(527,527);

	dialog_layout = new QGridLayout(this);
	
	
	make_label = new QLabel(this);
	dialog_layout->addWidget(make_label,0,0,1,1);
	
	d_make_edit = new QLineEdit(this);
	d_make_edit->setAlignment(Qt::AlignCenter);
	d_make_edit->setReadOnly(true);
	dialog_layout->addWidget(d_make_edit,0,1,1,2);
	
	model_label = new QLabel(this);
	dialog_layout->addWidget(model_label,1,0,1,1);
	
	d_model_edit = new QLineEdit(this);
	d_model_edit->setAlignment(Qt::AlignCenter);
	d_model_edit->setReadOnly(true);
	dialog_layout->addWidget(d_model_edit,1,1,1,2);
	
	option_label = new QLabel(this);
	dialog_layout->addWidget(option_label,2,0,1,1);

	d_option_edit = new QLineEdit(this);
	d_option_edit->setAlignment(Qt::AlignCenter);
	d_option_edit->setReadOnly(true);
	dialog_layout->addWidget(d_option_edit,2,1,1,2);
	
	separator_hline_1 = new QFrame(this);
	separator_hline_1->setFrameShape(QFrame::HLine);
	separator_hline_1->setFrameShadow(QFrame::Sunken);
	dialog_layout->addWidget(separator_hline_1,3,0,1,3);

	length_label = new QLabel(this);
	dialog_layout->addWidget(length_label,4,0,1,1);

	d_measurement_length_edit = new QLineEdit(this);
	d_measurement_length_edit->setAlignment(Qt::AlignCenter);
	dialog_layout->addWidget(d_measurement_length_edit,4,1,1,1);

	length_units_label = new QLabel(this);
	dialog_layout->addWidget(length_units_label,4,2,1,1);

	temperature_label = new QLabel(this);
	dialog_layout->addWidget(temperature_label,5,0,1,1);

	d_actual_temperature_edit = new QLineEdit(this);
	d_actual_temperature_edit->setAlignment(Qt::AlignCenter);
	dialog_layout->addWidget(d_actual_temperature_edit,5,1,1,1);

	temperature_units_label = new QLabel(this);
	dialog_layout->addWidget(temperature_units_label,5,2,1,1);
	
	separator_hline_2 = new QFrame(this);
	separator_hline_2->setFrameShape(QFrame::HLine);
	separator_hline_2->setFrameShadow(QFrame::Sunken);
	dialog_layout->addWidget(separator_hline_2,6,0,1,3);

	b89_repeat_label = new QLabel(this);
	dialog_layout->addWidget(b89_repeat_label,7,0,1,1);

	d_b89_repeat_edit = new QLineEdit(this);
	d_b89_repeat_edit->setAlignment(Qt::AlignCenter);
	d_b89_repeat_edit->setReadOnly(true);
	dialog_layout->addWidget(d_b89_repeat_edit,7,1,1,1);
	
	b89_repeat_description_label = new QLabel(this);
	dialog_layout->addWidget(b89_repeat_description_label,7,2,1,1);

	ballbar_range_label = new QLabel(this);
	dialog_layout->addWidget(ballbar_range_label,8,0,1,1);
	
	ballbar_range_description_label = new QLabel(this);
	dialog_layout->addWidget(ballbar_range_description_label,8,2,1,1);

	d_b89_volume_edit = new QLineEdit(this);
	d_b89_volume_edit->setAlignment(Qt::AlignCenter);
	d_b89_volume_edit->setReadOnly(true);
	dialog_layout->addWidget(d_b89_volume_edit,8,1,1,1);

	iso_r0_repeat_label = new QLabel(this);
	dialog_layout->addWidget(iso_r0_repeat_label,9,0,1,1);

	d_iso_r0_edit = new QLineEdit(this);
	d_iso_r0_edit->setAlignment(Qt::AlignCenter);
	d_iso_r0_edit->setReadOnly(true);
	dialog_layout->addWidget(d_iso_r0_edit,9,1,1,1);

	iso_r0_repeat_description_label = new QLabel(this);
	dialog_layout->addWidget(iso_r0_repeat_description_label,9,2,1,1);

	
	
	
	iso_e0_length_label = new QLabel(this);
	dialog_layout->addWidget(iso_e0_length_label,10,0,1,1);

	d_iso_e0_edit = new QLineEdit(this);
	d_iso_e0_edit->setAlignment(Qt::AlignCenter);
	d_iso_e0_edit->setReadOnly(true);
	dialog_layout->addWidget(d_iso_e0_edit,10,1,1,1);

	iso_e0_length_description_label = new QLabel(this);
	dialog_layout->addWidget(iso_e0_length_description_label,10,2,1,1);
	
	
	iso_e150_length_label = new QLabel(this);
	dialog_layout->addWidget(iso_e150_length_label,11,0,1,1);
	
	d_iso_e150_edit = new QLineEdit(this);
	d_iso_e150_edit->setAlignment(Qt::AlignCenter);
	d_iso_e150_edit->setReadOnly(true);
	dialog_layout->addWidget(d_iso_e150_edit,11,1,1,1);
	
	iso_e150_length_description_label = new QLabel(this);
	dialog_layout->addWidget(iso_e150_length_description_label,11,2,1,1);
	
	
	
	
	

	vdivde_v3_repeat_label = new QLabel(this);
	dialog_layout->addWidget(vdivde_v3_repeat_label,12,0,1,1);

	d_vdivde_v3_edit = new QLineEdit(this);
	d_vdivde_v3_edit->setAlignment(Qt::AlignCenter);
	d_vdivde_v3_edit->setReadOnly(true);
	dialog_layout->addWidget(d_vdivde_v3_edit,12,1,1,1);

	vdivde_v3_repeat_description_label = new QLabel(this);
	dialog_layout->addWidget(vdivde_v3_repeat_description_label,12,2,1,1);

	vdivde_u3_length_label = new QLabel(this);
	dialog_layout->addWidget(vdivde_u3_length_label,13,0,1,1);

	d_vdivde_u3_edit = new QLineEdit(this);
	d_vdivde_u3_edit->setAlignment(Qt::AlignCenter);
	d_vdivde_u3_edit->setReadOnly(true);
	dialog_layout->addWidget(d_vdivde_u3_edit,13,1,1,1);

	vdivde_u3_length_description_label = new QLabel(this);
	dialog_layout->addWidget(vdivde_u3_length_description_label,13,2,1,1);

	cmma_repeat_label = new QLabel(this);
	dialog_layout->addWidget(cmma_repeat_label,14,0,1,1);

	d_cmma_repeat_edit = new QLineEdit(this);
	d_cmma_repeat_edit->setAlignment(Qt::AlignCenter);
	d_cmma_repeat_edit->setReadOnly(true);
	dialog_layout->addWidget(d_cmma_repeat_edit,14,1,1,1);

	cmma_repeat_description_label = new QLabel(this);
	dialog_layout->addWidget(cmma_repeat_description_label,14,2,1,1);

	cmma_m_length_label = new QLabel(this);
	dialog_layout->addWidget(cmma_m_length_label,15,0,1,1);

	d_cmma_m_edit = new QLineEdit(this);
	d_cmma_m_edit->setAlignment(Qt::AlignCenter);
	d_cmma_m_edit->setReadOnly(true);
	dialog_layout->addWidget(d_cmma_m_edit,15,1,1,1);

	cmma_m_length_description_label = new QLabel(this);
	dialog_layout->addWidget(cmma_m_length_description_label,15,2,1,1);

	separator_hline_3 = new QFrame(this);
	separator_hline_3->setFrameShape(QFrame::HLine);
	separator_hline_3->setFrameShadow(QFrame::Sunken);
	dialog_layout->addWidget(separator_hline_3,16,0,1,3);

	dialog_vspacer = new QSpacerItem(0,0,QSizePolicy::Minimum,QSizePolicy::Expanding);
	dialog_layout->addItem(dialog_vspacer,17,0,1,3);
	
	button_hlayout = new QHBoxLayout();
	
	button_hspacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);
	button_hlayout->addItem(button_hspacer);
	
	close_button = new QPushButton(this);
	close_button->setAutoDefault(false);
	button_hlayout->addWidget(close_button);
	dialog_layout->addLayout(button_hlayout,187,0,1,3);


	// defaults
	d_temperature_deviation = 0.0;

	this->setWindowTitle(QStringLiteral("Specification Limits"));
	
	b89_repeat_description_label->setText(QStringLiteral("Sphere Repeatability"));
	length_units_label->setText(QStringLiteral("mm"));
	ballbar_range_label->setText(QStringLiteral("B89.4.1 Ballbar:"));
	model_label->setText(QStringLiteral("Model:"));
	length_label->setText(QStringLiteral("Measurement Length:"));
	make_label->setText(QStringLiteral("Make:"));
	ballbar_range_description_label->setText(QStringLiteral("Maximum Range"));
	option_label->setText(QStringLiteral("Option:"));
	temperature_label->setText(QStringLiteral("Actual Temperature:"));
	iso_r0_repeat_label->setText(QStringLiteral("10360-2 R0:"));
	iso_r0_repeat_description_label->setText(QStringLiteral("Length Repeatability"));
	iso_e0_length_label->setText(QStringLiteral("10360-2 E0:"));
	iso_e0_length_description_label->setText(QStringLiteral("Length Variation"));
	iso_e150_length_label->setText(QStringLiteral("10360-2 E150:"));
	iso_e150_length_description_label->setText(QStringLiteral("Length Variation with offset"));
	temperature_units_label->setText(QStringLiteral("C"));
	vdivde_v3_repeat_label->setText(QStringLiteral("VDIVDE 2617 V3:"));
	b89_repeat_label->setText(QStringLiteral("B89.4.1 Repeat:"));
	vdivde_v3_repeat_description_label->setText(QStringLiteral("3D Probe Variation"));
	vdivde_u3_length_label->setText(QStringLiteral("VDIVDE 2617 U3:"));
	vdivde_u3_length_description_label->setText(QStringLiteral("Length Variation"));
	cmma_repeat_label->setText(QStringLiteral("CMMA Repeat:"));
	cmma_repeat_description_label->setText(QStringLiteral("Length Repeatability"));
	cmma_m_length_label->setText(QStringLiteral("CMMA M:"));
	cmma_m_length_description_label->setText(QStringLiteral("Length Variation"));
	close_button->setText(QStringLiteral("Close"));
	
	d_measurement_length_edit->setText(QStringLiteral("1000.000"));
	d_actual_temperature_edit->setText(QStringLiteral("20.00"));
	
	connect(d_measurement_length_edit,&QLineEdit::editingFinished,this,&TSpecificationLimitsDialog::Length_Changed);
	connect(d_actual_temperature_edit,&QLineEdit::editingFinished,this,&TSpecificationLimitsDialog::Temperature_Changed);
	
	connect(close_button,&QPushButton::clicked,this,&TSpecificationLimitsDialog::close);
}

TSpecificationLimitsDialog::~TSpecificationLimitsDialog(void)
{
}

void TSpecificationLimitsDialog::Reset(
	const QString						&make,
	const QString						&model,
	const QString						&option)
{
	d_database_items.clear();
	
	d_make_edit->setText(make);
	d_model_edit->setText(model);
	d_option_edit->setText(option);
}

void TSpecificationLimitsDialog::Add_Database_Item(
	const TDatabaseItem					&item)
{
	d_database_items.push_back(item);
}

void TSpecificationLimitsDialog::Update(void)
{
	this->Update_Temperature_Deviation();
	this->Update_Limit_Values();
}

void TSpecificationLimitsDialog::Length_Changed(void)
{
	this->Update_Limit_Values();
}

void TSpecificationLimitsDialog::Temperature_Changed(void)
{
	this->Update_Temperature_Deviation();
	this->Update_Limit_Values();
}

void TSpecificationLimitsDialog::Update_Temperature_Deviation(void)
{
	double								dval;
	bool								valid;
	
	dval = d_actual_temperature_edit->text().toDouble(&valid);
	
	if(valid)
	{
		d_temperature_deviation = fabs(dval - 20.0);
	}
}

void TSpecificationLimitsDialog::Update_Limit_Values(void)
{
	double								length;
	double								expression_values[5];
	double								value;
	double								length_percentage_deviation;
	std::vector<TDatabaseItem>::const_iterator iter;
	TDatabaseItem::TSpecification_Item	specification_item;
	int									expression_type;
	int									count;
	int									cntr;
	bool								valid;
	static const double					MAXIMUM_LENGTH_VARIATION(0.20);	// +/- 20%
	static const double					MINIMUM_LENGTH(10.0);
	
	d_b89_repeat_edit->setText(QString());
	d_b89_volume_edit->setText(QString());
	d_iso_r0_edit->setText(QString());
	d_iso_e0_edit->setText(QString());
	d_iso_e150_edit->setText(QString());
	d_vdivde_v3_edit->setText(QString());
	d_vdivde_u3_edit->setText(QString());
	d_cmma_repeat_edit->setText(QString());
	d_cmma_m_edit->setText(QString());
	
	length = fabs(d_measurement_length_edit->text().toDouble(&valid));
	
	if(!valid)
	{
		return;
	}
	
	memset(expression_values,0,sizeof(expression_values));
	
	for(iter = d_database_items.begin();iter != d_database_items.end();++iter)
	{
		if((*iter).Standard_Type().compare(QStringLiteral("B89.4.1")) == 0)
		{
			count = (*iter).Specification_Items_Count();
			
			for(cntr = 0;cntr < count;++cntr)
			{
				specification_item = (*iter).Specification_Item(cntr);
				expression_type = specification_item.expression_item.Expression_Type();
				
				if(expression_type == TExpressionItem::EXPRESSION_VALUE &&
				   specification_item.component.compare(QStringLiteral("Repeat"))==0)
				{
					expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
					d_b89_repeat_edit->setText(QString("%1 um").arg(expression_values[0],0,'f',1));
				}
				
				if(specification_item.component.compare(QStringLiteral("Volume"))==0)
				{
					if(expression_type == TExpressionItem::EXPRESSION_VALUE_LENGTH)
					{
						expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
						expression_values[1] = specification_item.expression_item.Expression_Value(1,TExpressionItem::UNITS_MM);
						
						if(expression_values[1] > MINIMUM_LENGTH && length > MINIMUM_LENGTH)
						{
							value = length / expression_values[1];
							length_percentage_deviation = fabs(1.0 - value);
							
							if(length_percentage_deviation < MAXIMUM_LENGTH_VARIATION)
							{
								d_b89_volume_edit->setText(QString("%1 um").arg(expression_values[0],0,'f',1));
							}
							else
							{
								d_b89_volume_edit->setText(QString("%1/%2 um").arg(expression_values[0],0,'f',1).arg(expression_values[1]));
							}
						}
						else
						{
							d_b89_volume_edit->setText(QString("%1/%2 um").arg(expression_values[0],0,'f',1).arg(expression_values[1]));
						}
					}
					else if(expression_type == TExpressionItem::EXPRESSION_VALUE_LENGTH_TEMPERATURE)
					{
						expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
						expression_values[1] = specification_item.expression_item.Expression_Value(1,TExpressionItem::UNITS_UM);
						expression_values[2] = specification_item.expression_item.Expression_Value(2,TExpressionItem::UNITS_MM);
						
						if(expression_values[2] > MINIMUM_LENGTH && length > MINIMUM_LENGTH)
						{
							value = length / expression_values[2];
							length_percentage_deviation = fabs(1.0 - value);

							if(length_percentage_deviation < MAXIMUM_LENGTH_VARIATION)
							{
								value = expression_values[0];
								value += expression_values[1] * d_temperature_deviation;
																
								d_b89_volume_edit->setText(QString("%1 um").arg(value,0,'f',1));
							}
							else
							{
								d_b89_volume_edit->setText(QString("(%1+%2Td)/%3 um")
														   .arg(expression_values[0],0,'f',1)
														   .arg(expression_values[1],0,'f',2)
														   .arg(expression_values[2]));
							}
						}
						else
						{
							d_b89_volume_edit->setText(QString("(%1+%2Td)/%3 um")
													   .arg(expression_values[0],0,'f',1)
													   .arg(expression_values[1],0,'f',2)
													   .arg(expression_values[2]));
						}
					}
					else if(expression_type == TExpressionItem::EXPRESSION_CONST_VALUE_LENGTH)
					{
						expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
						expression_values[1] = specification_item.expression_item.Expression_Value(1,TExpressionItem::UNITS_UM);
						expression_values[2] = specification_item.expression_item.Expression_Value(2,TExpressionItem::UNITS_MM);

						if(expression_values[2] > MINIMUM_LENGTH)
						{
							value = expression_values[0];
							value += expression_values[1] * (1000.0 / expression_values[2]) * (length / 1000.0);
							
							d_b89_volume_edit->setText(QString("%1 um").arg(value,0,'f',1));
						}
					}

					if(expression_type == TExpressionItem::EXPRESSION_CONST_VALUE_LENGTH_LIMIT)
					{
						expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
						expression_values[1] = specification_item.expression_item.Expression_Value(1,TExpressionItem::UNITS_UM);
						expression_values[2] = specification_item.expression_item.Expression_Value(2,TExpressionItem::UNITS_MM);
						expression_values[3] = specification_item.expression_item.Expression_Value(3,TExpressionItem::UNITS_UM);
						
						if(expression_values[2] > MINIMUM_LENGTH)
						{
							value = expression_values[0];
							value += expression_values[1] * (1000.0 / expression_values[2]) * (length / 1000.0);
							
							if(value > expression_values[3])
							{
								value = expression_values[3];
							}
							
							d_b89_volume_edit->setText(QString("%1 um").arg(value,0,'f',1));
						}
					}
					else if(expression_type == TExpressionItem::EXPRESSION_CONST_VALUE_LENGTH_TEMPERATURE)
					{
						expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
						expression_values[1] = specification_item.expression_item.Expression_Value(1,TExpressionItem::UNITS_UM);
						expression_values[2] = specification_item.expression_item.Expression_Value(2,TExpressionItem::UNITS_UM);
						expression_values[3] = specification_item.expression_item.Expression_Value(3,TExpressionItem::UNITS_UM);
						expression_values[4] = specification_item.expression_item.Expression_Value(4,TExpressionItem::UNITS_MM);

						if(expression_values[4] > MINIMUM_LENGTH)
						{
							value = expression_values[0];
							value += expression_values[1] * d_temperature_deviation;
							
							value += (expression_values[2] + expression_values[3] * d_temperature_deviation) * (1000.0 / expression_values[4]) * (length / 1000.0);
							
							d_b89_volume_edit->setText(QString("%1 um").arg(value,0,'f',1));
						}
					}
				}
			}
		}
		else if((*iter).Standard_Type().compare(QStringLiteral("10360")) == 0)
		{
			count = (*iter).Specification_Items_Count();
			
			for(cntr = 0;cntr < count;++cntr)
			{
				specification_item = (*iter).Specification_Item(cntr);
				expression_type = specification_item.expression_item.Expression_Type();
				
				if(expression_type == TExpressionItem::EXPRESSION_VALUE &&
				   specification_item.component.compare(QStringLiteral("R0"))==0)
				{
					expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
					d_iso_r0_edit->setText(QString("%1 um").arg(expression_values[0],0,'f',1));
				}
				
				if(specification_item.component.compare(QStringLiteral("E0"))==0)
				{
					if(expression_type == TExpressionItem::EXPRESSION_CONST_VALUE_LENGTH)
					{
						expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
						expression_values[1] = specification_item.expression_item.Expression_Value(1,TExpressionItem::UNITS_UM);
						expression_values[2] = specification_item.expression_item.Expression_Value(2,TExpressionItem::UNITS_MM);
						
						if(expression_values[2] > MINIMUM_LENGTH)
						{
							value = expression_values[0];
							value += expression_values[1] * (1000.0 / expression_values[2]) * (length / 1000.0);
							
							d_iso_e0_edit->setText(QString("+/- %1 um").arg(value,0,'f',1));
						}
					}
					else if(expression_type == TExpressionItem::EXPRESSION_CONST_VALUE_LENGTH_LIMIT)
					{
						expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
						expression_values[1] = specification_item.expression_item.Expression_Value(1,TExpressionItem::UNITS_UM);
						expression_values[2] = specification_item.expression_item.Expression_Value(2,TExpressionItem::UNITS_MM);
						expression_values[3] = specification_item.expression_item.Expression_Value(3,TExpressionItem::UNITS_UM);

						if(expression_values[2] > MINIMUM_LENGTH)
						{
							value = expression_values[0];
							value += expression_values[1] * (1000.0 / expression_values[2]) * (length / 1000.0);
							
							if(value > expression_values[3])
							{
								value = expression_values[3];
							}
							
							d_iso_e0_edit->setText(QString("+/- %1 um").arg(value,0,'f',1));
						}
					}
					else if(expression_type == TExpressionItem::EXPRESSION_CONST_VALUE_LENGTH_TEMPERATURE)
					{
						expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
						expression_values[1] = specification_item.expression_item.Expression_Value(1,TExpressionItem::UNITS_UM);
						expression_values[2] = specification_item.expression_item.Expression_Value(2,TExpressionItem::UNITS_UM);
						expression_values[3] = specification_item.expression_item.Expression_Value(3,TExpressionItem::UNITS_UM);
						expression_values[4] = specification_item.expression_item.Expression_Value(4,TExpressionItem::UNITS_MM);
						
						if(expression_values[4] > MINIMUM_LENGTH)
						{
							value = expression_values[0];
							value += expression_values[1] * d_temperature_deviation;
							
							value += (expression_values[2] + expression_values[3] * d_temperature_deviation) * (1000.0 / expression_values[4]) * (length / 1000.0);
							
							d_iso_e0_edit->setText(QString("+/- %1 um").arg(value,0,'f',1));
						}
					}
				}
				
				if(specification_item.component.compare(QStringLiteral("E150"))==0)
				{
					if(expression_type == TExpressionItem::EXPRESSION_CONST_VALUE_LENGTH)
					{
						expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
						expression_values[1] = specification_item.expression_item.Expression_Value(1,TExpressionItem::UNITS_UM);
						expression_values[2] = specification_item.expression_item.Expression_Value(2,TExpressionItem::UNITS_MM);

						if(expression_values[2] > MINIMUM_LENGTH)
						{
							value = expression_values[0];
							value += expression_values[1] * (1000.0 / expression_values[2]) * (length / 1000.0);

							d_iso_e150_edit->setText(QString("+/- %1 um").arg(value,0,'f',1));
						}
					}
					else if(expression_type == TExpressionItem::EXPRESSION_CONST_VALUE_LENGTH_LIMIT)
					{
						expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
						expression_values[1] = specification_item.expression_item.Expression_Value(1,TExpressionItem::UNITS_UM);
						expression_values[2] = specification_item.expression_item.Expression_Value(2,TExpressionItem::UNITS_MM);
						expression_values[3] = specification_item.expression_item.Expression_Value(3,TExpressionItem::UNITS_UM);

						if(expression_values[2] > MINIMUM_LENGTH)
						{
							value = expression_values[0];
							value += expression_values[1] * (1000.0 / expression_values[2]) * (length / 1000.0);
							
							if(value > expression_values[3])
							{
								value = expression_values[3];
							}
							
							d_iso_e150_edit->setText(QString("+/- %1 um").arg(value,0,'f',1));
						}
					}
					else if(expression_type == TExpressionItem::EXPRESSION_CONST_VALUE_LENGTH_TEMPERATURE)
					{
						expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
						expression_values[1] = specification_item.expression_item.Expression_Value(1,TExpressionItem::UNITS_UM);
						expression_values[2] = specification_item.expression_item.Expression_Value(2,TExpressionItem::UNITS_UM);
						expression_values[3] = specification_item.expression_item.Expression_Value(3,TExpressionItem::UNITS_UM);
						expression_values[4] = specification_item.expression_item.Expression_Value(4,TExpressionItem::UNITS_MM);
						
						if(expression_values[4] > MINIMUM_LENGTH)
						{
							value = expression_values[0];
							value += expression_values[1] * d_temperature_deviation;
							
							value += (expression_values[2] + expression_values[3] * d_temperature_deviation) * (1000.0 / expression_values[4]) * (length / 1000.0);
							
							d_iso_e150_edit->setText(QString("+/- %1 um").arg(value,0,'f',1));
						}
					}
				}
			}
		}
		else if((*iter).Standard_Type().compare(QStringLiteral("VDIVDE_2617")) == 0)
		{
			count = (*iter).Specification_Items_Count();
			
			for(cntr = 0;cntr < count;++cntr)
			{
				specification_item = (*iter).Specification_Item(cntr);
				expression_type = specification_item.expression_item.Expression_Type();
				
				if(expression_type == TExpressionItem::EXPRESSION_VALUE &&
				   specification_item.component.compare(QStringLiteral("V3"))==0)
				{
					expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
					d_vdivde_v3_edit->setText(QString("%1 um").arg(expression_values[0],0,'f',1));
				}
				
				if(specification_item.component.compare(QStringLiteral("U3"))==0)
				{
					if(expression_type == TExpressionItem::EXPRESSION_CONST_VALUE_LENGTH)
					{
						expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
						expression_values[1] = specification_item.expression_item.Expression_Value(1,TExpressionItem::UNITS_UM);
						expression_values[2] = specification_item.expression_item.Expression_Value(2,TExpressionItem::UNITS_MM);
	
						if(expression_values[2] > MINIMUM_LENGTH)
						{
							value = expression_values[0];
							value += expression_values[1] * (1000.0 / expression_values[2]) * (length / 1000.0);
							
							d_vdivde_u3_edit->setText(QString("+/- %1 um").arg(value,0,'f',1));
						}
					}
					else if(expression_type == TExpressionItem::EXPRESSION_CONST_VALUE_LENGTH_LIMIT)
					{
						expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
						expression_values[1] = specification_item.expression_item.Expression_Value(1,TExpressionItem::UNITS_UM);
						expression_values[2] = specification_item.expression_item.Expression_Value(2,TExpressionItem::UNITS_MM);
						expression_values[3] = specification_item.expression_item.Expression_Value(3,TExpressionItem::UNITS_UM);

						if(expression_values[2] > MINIMUM_LENGTH)
						{
							value = expression_values[0];
							value += expression_values[1] * (1000.0 / expression_values[2]) * (length / 1000.0);
							
							if(value > expression_values[3])
							{
								value = expression_values[3];
							}
							
							d_vdivde_u3_edit->setText(QString("+/- %1 um").arg(value,0,'f',1));
						}
					}
					else if(expression_type == TExpressionItem::EXPRESSION_CONST_VALUE_LENGTH_TEMPERATURE)
					{
						expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
						expression_values[1] = specification_item.expression_item.Expression_Value(1,TExpressionItem::UNITS_UM);
						expression_values[2] = specification_item.expression_item.Expression_Value(2,TExpressionItem::UNITS_UM);
						expression_values[3] = specification_item.expression_item.Expression_Value(3,TExpressionItem::UNITS_UM);
						expression_values[4] = specification_item.expression_item.Expression_Value(4,TExpressionItem::UNITS_MM);
						
						if(expression_values[4] > MINIMUM_LENGTH)
						{
							value = expression_values[0];
							value += expression_values[1] * d_temperature_deviation;
							
							value += (expression_values[2] + expression_values[3] * d_temperature_deviation) * (1000.0 / expression_values[4]) * (length / 1000.0);
							
							d_vdivde_u3_edit->setText(QString("+/- %1 um").arg(value,0,'f',1));
						}
					}
				}
			}
		}
		else if((*iter).Standard_Type().compare(QStringLiteral("CMMA")) == 0)
		{
			count = (*iter).Specification_Items_Count();
			
			for(cntr = 0;cntr < count;++cntr)
			{
				specification_item = (*iter).Specification_Item(cntr);
				expression_type = specification_item.expression_item.Expression_Type();
				
				if(expression_type == TExpressionItem::EXPRESSION_VALUE &&
				   specification_item.component.compare(QStringLiteral("Repeat"))==0)
				{
					expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
					d_cmma_repeat_edit->setText(QString("%1 um").arg(expression_values[0],0,'f',1));
				}
				
				if(specification_item.component.compare(QStringLiteral("M"))==0)
				{
					if(expression_type == TExpressionItem::EXPRESSION_CONST_VALUE_LENGTH)
					{
						expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
						expression_values[1] = specification_item.expression_item.Expression_Value(1,TExpressionItem::UNITS_UM);
						expression_values[2] = specification_item.expression_item.Expression_Value(2,TExpressionItem::UNITS_MM);
						
						if(expression_values[2] > MINIMUM_LENGTH)
						{
							value = expression_values[0];
							value += expression_values[1] * (1000.0 / expression_values[2]) * (length / 1000.0);
							
							d_cmma_m_edit->setText(QString("+/- %1 um").arg(value,0,'f',1));
						}
					}
					else if(expression_type == TExpressionItem::EXPRESSION_CONST_VALUE_LENGTH_LIMIT)
					{
						expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
						expression_values[1] = specification_item.expression_item.Expression_Value(1,TExpressionItem::UNITS_UM);
						expression_values[2] = specification_item.expression_item.Expression_Value(2,TExpressionItem::UNITS_MM);
						expression_values[3] = specification_item.expression_item.Expression_Value(3,TExpressionItem::UNITS_UM);

						if(expression_values[2] > MINIMUM_LENGTH)
						{
							value = expression_values[0];
							value += expression_values[1] * (1000.0 / expression_values[2]) * (length / 1000.0);
							
							if(value > expression_values[3])
							{
								value = expression_values[3];
							}
							
							d_cmma_m_edit->setText(QString("+/- %1 um").arg(value,0,'f',1));
						}
					}
					else if(expression_type == TExpressionItem::EXPRESSION_CONST_VALUE_LENGTH_TEMPERATURE)
					{
						expression_values[0] = specification_item.expression_item.Expression_Value(0,TExpressionItem::UNITS_UM);
						expression_values[1] = specification_item.expression_item.Expression_Value(1,TExpressionItem::UNITS_UM);
						expression_values[2] = specification_item.expression_item.Expression_Value(2,TExpressionItem::UNITS_UM);
						expression_values[3] = specification_item.expression_item.Expression_Value(3,TExpressionItem::UNITS_UM);
						expression_values[4] = specification_item.expression_item.Expression_Value(4,TExpressionItem::UNITS_MM);
						
						if(expression_values[4] > MINIMUM_LENGTH)
						{
							value = expression_values[0];
							value += expression_values[1] * d_temperature_deviation;
							
							value += (expression_values[2] + expression_values[3] * d_temperature_deviation) * (1000.0 / expression_values[4]) * (length / 1000.0);
							
							d_cmma_m_edit->setText(QString("+/- %1 um").arg(value,0,'f',1));
						}
					}
				}
			}
		}
	}
}

