/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#ifndef SPECIFICATIONLIMITSDIALOGHEADERFILE
#define SPECIFICATIONLIMITSDIALOGHEADERFILE

#include <QDialog>
#include <vector>

#include "databaseitem.h"
#include "expressionitem.h"

class QLineEdit;

class TSpecificationLimitsDialog : public QDialog
{
Q_OBJECT

public:

// CREATORS
	TSpecificationLimitsDialog(const QWidget *parent=0, Qt::WindowFlags flags=Qt::WindowFlags());
	~TSpecificationLimitsDialog(void);

// ACCESSORS

// MANIPULATORS
	void Reset(const QString &make,const QString &model,const QString &option);
	void Add_Database_Item(const TDatabaseItem &item);
	void Update(void);

signals:

private slots:
	
	void Length_Changed(void);
	void Temperature_Changed(void);

private:
	QLineEdit							*d_make_edit;
	QLineEdit							*d_model_edit;
	QLineEdit							*d_option_edit;
	QLineEdit							*d_measurement_length_edit;
	QLineEdit							*d_actual_temperature_edit;
	QLineEdit							*d_b89_repeat_edit;
	QLineEdit							*d_b89_volume_edit;
	QLineEdit							*d_iso_r0_edit;
	QLineEdit							*d_iso_e0_edit;
	QLineEdit							*d_iso_e150_edit;
	QLineEdit							*d_vdivde_v3_edit;
	QLineEdit							*d_vdivde_u3_edit;
	QLineEdit							*d_cmma_repeat_edit;
	QLineEdit							*d_cmma_m_edit;
	
	std::vector<TDatabaseItem>			d_database_items;
	double								d_temperature_deviation;
	
	void Update_Temperature_Deviation(void);
	void Update_Limit_Values(void);
	
// NOT IMPLEMENTED
	TSpecificationLimitsDialog(const TSpecificationLimitsDialog&);
	TSpecificationLimitsDialog& operator=(const TSpecificationLimitsDialog&);
};

#endif
